#
# app.make
#
# Variable definitions and rules for building application projects.  An
# application is a directory which contains an executable and any resources
# that executable requires.  See wrapped.make for more information about
# projects whose product is a directory.
#
# PUBLIC TARGETS
#    app: synonymous with all
#
# IMPORTED VARIABLES
#    APP_WRAPPER_EXTENSION:  the extension for the app wrapper.  Defaults to ".app"
#
# EXPORTED VARIABLES
#    none
#

.PHONY: app all
app: all
PROJTYPE = APP

ifeq "" "$(APP_WRAPPER_EXTENSION)"
APP_WRAPPER_EXTENSION = .app
endif

PRODUCT = $(PRODUCT_DIR)/$(NAME)$(APP_WRAPPER_EXTENSION)
PRODUCTS = $(PRODUCT)
PROJTYPE_GARBAGE = $(PRODUCT_DIR)/$(NAME).debug $(PRODUCT_DIR)/$(NAME).profile
INNER_PRODUCT = $(PRODUCT)/$(NAME)$(EXECUTABLE_EXT)
STRIPPED_PRODUCTS = $(INNER_PRODUCT)

ifeq "WINDOWS" "$(OS)"
REG_FILE = appResources.reg
RESOURCE_OFILE = appResources.o
PROJTYPE_LDFLAGS = -win
PROJTYPE_OFILES = $(RESOURCE_OFILE)
PROJTYPE_LIBS = $(NEXT_ROOT)/NextDeveloper/Libraries/libNSWinMain.a
endif
ifeq "NEXTSTEP" "$(OS)"
PROJTYPE_LDFLAGS = -sectcreate __ICON __header $(NAME).iconheader -segprot __ICON r r $(ICONSECTIONS)
endif

include $(MAKEFILEDIR)/wrapped-common.make
-include $(LOCAL_MAKEFILEDIR)/app.make.preamble

IMPLICIT_SOURCE_FILES += $(NAME).iconheader

$(PRODUCT): $(INNER_PRODUCT)

$(INNER_PRODUCT): $(DEPENDENCIES)
	$(SILENT) $(MKDIRS) $(PRODUCT)
	$(CC) $(ALL_LDFLAGS) $(ARCHITECTURE_FLAGS) -o $(INNER_PRODUCT) $(LOADABLES)

-include $(LOCAL_MAKEFILEDIR)/app.make.postamble

