#
# wrapped-common.make
#
# Rules and variables common to all projects with wrapped-style products
# (i.e. applications, bundles, etc.).  Projects which include this file
# are assumed to have defined the PRODUCT variable to be the path to the
# directory and the INNER_PRODUCT to be the path to the executable within
# the directory.  Wrapped products consist of a directory containing the
# executable, a Resources, Headers, and PrivateHeaders directories.
#
# IMPORTED VARIABLES
#    none
#
# EXPORTED VARIABLES
#    none
#

WRAPPED = YES

# The resource directories are exported so that non-wrapped
# subprojects can copy their resources.
export GLOBAL_RESOURCE_DIR = $(PRODUCT)$(VERSION_SUFFIX)/Resources
export LOCAL_RESOURCE_DIR = $(PRODUCT)$(VERSION_SUFFIX)/Resources/$(LANGUAGE).lproj
INFO_FILE = $(GLOBAL_RESOURCE_DIR)/Info-$(PLATFORM_OS).plist
USER_INFO_FILE_NAME = CustomInfo.plist
HELP_FILE = $(GLOBAL_RESOURCE_DIR)/Help.plist
BEFORE_BUILD += create-info-file create-help-file

include $(MAKEFILEDIR)/common.make
-include $(LOCAL_MAKEFILEDIR)/wrapped-common.make.preamble

#
# prebuild rules
#

.PHONY: create-info-file

create-info-file: $(GLOBAL_RESOURCE_DIR) $(INFO_FILE)

ifneq "$(REG_APPEND_FILE)" ""
REG_APPEND_FLAGS = -append $(REG_APPEND_FILE)
endif

ifneq "$(REG_FILE)" ""
$(RESOURCE_OFILE) $(GLOBAL_RESOURCE_DIR)/$(REG_FILE): $(GLOBAL_RESOURCE_DIR) $(OFILE_DIR) PB.project $(WINDOWS_APPICON) $(WINDOWS_DOCICONS)
	$(REGGEN) -o $(OFILE_DIR)/$(RESOURCE_OFILE) $(REG_APPEND_FLAGS) -project PB.project -rc $(RC_CMD) -regFile $(GLOBAL_RESOURCE_DIR)/$(REG_FILE)
endif
  	

$(INFO_FILE): $(GLOBAL_RESOURCE_DIR) PB.project $(OTHER_INFO_FILES)
ifneq "$(ENABLE_INFO_DICTIONARY)" "NO"
	$(SILENT) if [ -r "$(USER_INFO_FILE_NAME)" ] ; then \
	   mergeInfoArgs="$(USER_INFO_FILE_NAME)" ; \
	fi ; \
	if [ -n "$(REG_FILE)" ] ; then \
	   mergeInfoArgs="$$mergeInfoArgs -regFile $(REG_FILE)" ; \
	fi ; \
	if [ -x "$(MERGEINFO)$(EXECUTABLE_EXT)" ] ; then \
	   $(RM) -f $(INFO_FILE) ; \
	   cmd="$(MERGEINFO) PB.project $$mergeInfoArgs $(OTHER_INFO_FILES) -o $(INFO_FILE)" ; \
	   $(ECHO) $$cmd ; eval $$cmd ; \
	fi
endif

create-help-file: $(GLOBAL_RESOURCE_DIR) $(HELP_FILE)

$(HELP_FILE): $(HELP_FILES)
ifneq "" "$(COMPILEHELP)"
ifneq "$(ENABLE_HELP_DICTIONARY)" "NO"
	$(COMPILEHELP) -f $(HELP_FILES) -o $(HELP_FILE)
endif
endif

-include $(LOCAL_MAKEFILEDIR)/wrapped-common.make.postamble
